from GuiComp import ImageScraperSource, CompWorkspace, lemonimg, lrfimg, slsimg
from threading import Thread, Lock
from cherrypy import engine
import time

class CAFDataProvider(Thread): 
  def __init__(self, datadir):
    Thread.__init__(self, name="CAFDataProvider")  
    self.lock = Lock()
    self.stopme = False
    self.datadir = datadir
    self.data = {'u_cmsalca_datasets_order_date': '',
		 'u_cmsalca_datasets_order_size': '',
		 'u_cmscomm_datasets_order_date': '',
		 'u_cmscomm_datasets_order_size': '',
		 'u_cmsphys_datasets_order_date': '',
		 'u_cmsphys_datasets_order_size': ''}
    self.reset()

  def reset(self):
    for fname in self.data:
      self.data[fname] = "Data not found, please report CAF monitoring problem."
    self.alca_percent = "CMSALCA percetages not found, please report CAF monitoring problem."
    self.comm_percent = "CMSCOMM percetages not found, please report CAF monitoring problem."
    self.phys_percent = "CMSPHYS percetages not found, please report CAF monitoring problem."

  def stop(self):
    self.lock.acquire()
    self.stopme = True
    self.lock.release()

  # Run forever retrieving CAF, reading CAF statistics every 30 minutes.
  # Only complication here is we want to exit early on server shutdown.
  def run(self):
    while True:
      self.lock.acquire()
      stopme = self.stopme
      self.lock.release()

      if stopme:
        break

      self.reset()
      try:
        f = file(self.datadir + "/percent_totals_stakeholder")
        self.alca_percent = f.readline()
        self.comm_percent = f.readline()
        self.phys_percent = f.readline()
        f.close()

        for fname in self.data:
          self.data[fname] = file("%s/%s" % (self.datadir, fname)).read()
      except IOError:
	pass

      for w in xrange(1, 10):
        self.lock.acquire()
        stopme = self.stopme
        self.lock.release()

        if stopme:
	  break

	time.sleep(1)

# Source to print CAF status text files.
class CAFDiskSource:
  plothook = 'caf-disk'
  def __init__(self, server, statedir, *args):
    self.thread = CAFDataProvider(statedir)
    engine.subscribe('stop', self.thread.stop)
    self.thread.start()

  def plot(self, group, *args, **kwargs):
    if group == 'alcabydate':
      return 'text/plain', self.thread.data['u_cmsalca_datasets_order_date']
    elif group == 'alcabysize':
      return 'text/plain', self.thread.data['u_cmsalca_datasets_order_size']
    elif group == 'commbydate':
      return 'text/plain', self.thread.data['u_cmscomm_datasets_order_date']
    elif group == 'commbysize':
      return 'text/plain', self.thread.data['u_cmscomm_datasets_order_size']
    elif group == 'physbydate':
      return 'text/plain', self.thread.data['u_cmsphys_datasets_order_date']
    elif group == 'physbysize':
      return 'text/plain', self.thread.data['u_cmsphys_datasets_order_size']
    elif group == 'alcapercent':
      return 'text/plain', self.thread.alca_percent
    elif group == 'commpercent':
      return 'text/plain', self.thread.comm_percent
    elif group == 'physpercent':
      return 'text/plain', self.thread.phys_percent
    else:
      return None, None

class CAFPlotSource(ImageScraperSource):
  def __init__(self, server, dir, *args):
    ImageScraperSource.__init__(self, 'caf-plot', [
      lemonimg("web", "info", ["lxbatch-cmscaf-cpu", "lxbatch-cmscaf-net"], "/lemon-web/images/", entity="lxbatch/cmscaf", time=0, offset=0),
      lemonimg("web", "info", ["lxbatch-cmsexpress-cpu", "lxbatch-cmsexpress-net"], "/lemon-web/images/", entity="lxbatch/cmsexpress", time=0, offset=0),
      lemonimg("web", "info", ["lxbatch-cmsinter-cpu", "lxbatch-cmsinter-net"], "/lemon-web/images/", entity="lxbatch/cmsinter", time=0, offset=0),
      lemonimg("web", "metric_distribution", ["lxbatch-cmscaf-loadavg"], "cache/", entity="lxbatch/cmscaf", metric="LoadAvg", field="LoadAvg"),
      lemonimg("web", "metric_distribution", ["lxbatch-cmsexpress-loadavg"], "cache/", entity="lxbatch/cmsexpress", metric="LoadAvg", field="LoadAvg"),
      lemonimg("web", "metric_distribution", ["lxbatch-cmsinter-loadavg"], "cache/", entity="lxbatch/cmsinter", metric="LoadAvg", field="LoadAvg"),
      lemonimg("status", "info", ["c2cms-cmscaf-cpu", "c2cms-cmscaf-net"], "/lemon-status/images/", entity="c2cms/cmscaf", time="0.0.5", offset=0),
      lemonimg("status", "info", ["c2cms-cmscafuser-cpu", "c2cms-cmscafuser-net"], "/lemon-status/images/", entity="c2cms/cmscafuser", time="0.0.5", offset=0),
      lemonimg("status", "metric_distribution", ["c2cms-cmscaf-loadavg"], "cache/", entity="c2cms/cmscaf", metric="LoadAvg", field="LoadAvg"),
      lemonimg("status", "metric_distribution", ["c2cms-cmscafuser-loadavg"], "cache/", entity="c2cms/cmscafuser", metric="LoadAvg", field="LoadAvg"),
      slsimg(["avail"], "graph.php", id="CASTORCMS_CMSCAF", more="availability", period="day"),
      slsimg(["disk-free"], "graph.php", id="CASTORCMS_CMSCAF", more="nv:Percentage Free Space", period="week"),
      slsimg(["diskuser-free"], "graph.php", id="CASTORCMS_CMSCAFUSER", more="nv:Percentage Free Space", period="week"),
      lrfimg(["job-queue-caf1nh"], "rrd_RUN_d.gif", queue="cmscaf1nh"),
      lrfimg(["job-queue-caf1nd"], "rrd_RUN_d.gif", queue="cmscaf1nd"),
      lrfimg(["job-queue-caf1nw"], "rrd_RUN_d.gif", queue="cmscaf1nw"),
      lrfimg(["job-queue-cmsexpress"], "rrd_RUN_d.gif", queue="cmsexpress")])

class CompCAFWorkspace(CompWorkspace):
  class StorageSummary:
    name = "Storage pools performance"
    def state(self, session):
      return "{kind: 'CompCAFCastor', version: %d}" % time.time()

  class BatchSummary:
    name = "LSF performance"
    def state(self, session):
      return "{kind: 'CompCAFBatch', version: %d}" % time.time()

  class JobSummary:
    name = "Job summary"
    def state(self, session):
      return "{kind: 'CompCAFJobs', version: %d}" % time.time()

  class DiskMonitor:
    name = "Disk monitoring"
    def state(self, session):
      return "{kind: 'CompCAFDisk', version: %d}" % time.time()

  def __init__(self, gui, rank, category, name, *args):
    CompWorkspace.__init__(self, gui, rank, category, 'caf', name,
			   [self.StorageSummary(), self.BatchSummary(),
			    self.JobSummary(), self.DiskMonitor()])
    gui._addJSFragment("%s/gui-comp-caf.js" % gui.contentpath)

  def _state(self, session):
    return self._dostate(session, "CAF_workflows_monitoring")
